@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX add-in functions.
-;;; yatexadd.el rev.14
-;;; (c )1991-2000 by HIROSE Yuuji.[yuuji@yatex.org]
-;;; Last modified Mon Dec 25 19:17:09 2000 on firestorm
-;;; $Id: yatexadd.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
+;;; yatexadd.el rev.15
+;;; (c )1991-2003 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Mon Mar 17 16:46:01 2003 on serow
+;;; $Id: yatexadd.el,v 1.71 2003/05/01 13:38:49 yuuji Rel $
 
 ;;;
 ;;Sample functions for LaTeX environment.
@@ -90,9 +90,17 @@ YaTeX-make-begin-end."
 
 (defun YaTeX:table ()
   "YaTeX add-in function for table environment."
-  (setq YaTeX-env-name "tabular"
-	YaTeX-section-name "caption")
-  (YaTeX:read-position "htbp"))
+  (cond
+   ((eq major-mode 'yatex-mode)
+    (setq YaTeX-env-name "tabular"
+	  YaTeX-section-name "caption")
+    (YaTeX:read-position "htbp"))
+   ((eq major-mode 'texinfo-mode)
+    (concat " "
+	    (completing-read
+	     "Highlights with: "
+	     '(("@samp")("@kbd")("@code")("@asis")("@file")("@var"))
+	     nil nil "@")))))
 
 (fset 'YaTeX:figure 'YaTeX:table)
 (fset 'YaTeX:figure* 'YaTeX:table)
@@ -136,7 +144,7 @@ YaTeX-make-begin-end."
 
 (defun YaTeX:thebibliography ()
   (setq YaTeX-section-name "bibitem")
-  "")
+  (concat "{" (read-string "Longest label: ") "}"))
 
 ;;;
 ;;Sample functions for section-type command.
@@ -273,8 +281,12 @@ YaTeX-make-begin-end."
       (concat "[" label "]"))))
 
 (defun YaTeX:item ()
-  (YaTeX-indent-line)
-  (setq YaTeX-section-name "label")
+  (cond
+   ((eq major-mode 'yatex-mode)
+    (YaTeX-indent-line)
+    (setq YaTeX-section-name "label"))
+   ((eq major-mode 'texinfo-mode)
+    (setq YaTeX-section-name "dots"))) ;??
   " ")
 (fset 'YaTeX:item\[\] 'YaTeX:item)
 (fset 'YaTeX:subitem 'YaTeX:item)
@@ -464,8 +476,11 @@ YaTeX-make-begin-end."
 (defun YaTeX::ref-getset-label (buffer point)
   "Get label string in the BUFFER near the POINT.
 Make \\label{xx} if no label."
-  (let (boundary inspoint cc newlabel (labelholder "label") mathp env)
-    ;(set-buffer buffer)
+  ;;Here, we rewrite the LaTeX source.  Therefore we should be careful
+  ;;to decide the location suitable for \label.  Do straightforward!
+  (let (boundary inspoint cc newlabel (labelholder "label") mathp env
+       (r-escape (regexp-quote YaTeX-comment-prefix)))
+    ;;(set-buffer buffer)
     (switch-to-buffer buffer)
     (save-excursion
       (goto-char point)
@@ -476,29 +491,84 @@ Make \\label{xx} if no label."
 	(skip-chars-forward "^{")
 	(forward-list 1)
 	(skip-chars-forward " \t\n")
-	(setq boundary "[^\\]"))
+	;(setq boundary "[^\\]")
+	(setq inspoint (point))
+	(setq boundary
+	      (save-excursion
+		(if (YaTeX-re-search-active-forward
+		     (concat YaTeX-ec-regexp
+			     "\\(" YaTeX-sectioning-regexp "\\|"
+			     "begin\\|item\\)")
+		     r-escape nil 1)
+		    (match-beginning 0)
+		  (1- (point))))))
        ((looking-at "item\\s ")
 	(setq cc (+ cc 6))
-	(setq boundary (concat YaTeX-ec-regexp "\\(item\\|begin\\|end\\)\\b")))
+	;(setq boundary (concat YaTeX-ec-regexp "\\(item\\|begin\\|end\\)\\b"))
+	(setq boundary
+	      (save-excursion
+		(if (YaTeX-re-search-active-forward
+		     (concat YaTeX-ec-regexp "\\(item\\|begin\\|end\\)\\b")
+		     r-escape nil 1)
+		    (match-beginning 0)
+		  (1- (point))))
+	      inspoint boundary))
        ((looking-at "bibitem")
-	(setq labelholder "bibitem"))	; label holder is bibitem itself
+	(setq labelholder "bibitem")	; label holder is bibitem itself
+	(setq boundary
+	      (save-excursion
+		(if (YaTeX-re-search-active-forward
+		     (concat YaTeX-ec-regexp "\\(bibitem\\|end\\)\\b")
+		     r-escape nil 1)
+		    (match-beginning 0)
+		  (1- (point))))
+	      inspoint boundary))
        ((string-match YaTeX::ref-mathenv-regexp
 		      (setq env (or (YaTeX-inner-environment t) "document")))
 	(setq mathp t)
-	(setq boundary (concat YaTeX-ec-regexp "\\(\\\\\\|end{" env "}\\)")))
+	;;(setq boundary (concat YaTeX-ec-regexp "\\(\\\\\\|end{" env "}\\)"))
+	(setq boundary
+	      (save-excursion
+		(if (YaTeX-re-search-active-forward
+		     (concat YaTeX-ec-regexp "\\(\\\\\\|end{" env "}\\)")
+		     r-escape nil 1)
+		    (match-beginning 0)
+		  (1- (point))))
+	      inspoint boundary))
+       ((looking-at "footnote\\s *{")
+	(skip-chars-forward "^{")	;move onto `{'
+	(setq boundary
+	      (save-excursion
+		(condition-case err
+		    (forward-list 1)
+		  (error (error "\\\\footnote at point %s's brace not closed"
+				(point))))
+		(1- (point)))
+	      inspoint boundary))
        ((looking-at "caption\\|\\(begin\\)")
 	(skip-chars-forward "^{")
-	(if (match-beginning 1) (forward-list 1))
-	(setq boundary (concat YaTeX-ec-regexp "\\(begin\\|end\\)\\b")))
+	;;;;;;(if (match-beginning 1) (forward-list 1))
+	;; caption can be treated as mathenv, is it right??
+	(forward-list 1)
+	;;(setq boundary (concat YaTeX-ec-regexp "\\(begin\\|end\\)\\b"))
+	(setq inspoint (point))
+	(setq boundary
+	      (save-excursion
+		(if (YaTeX-re-search-active-forward
+		     (concat YaTeX-ec-regexp "\\(begin\\|end\\)\\b")
+		     r-escape nil 1)
+		    (match-beginning 0)
+		(1- (point))))))
        (t ))
       (if (save-excursion (skip-chars-forward " \t") (looking-at "%"))
 	  (forward-line 1))
       (if (and (save-excursion
 		 (YaTeX-re-search-active-forward
-		  (concat "\\(" labelholder "\\)\\|\\(" boundary "\\)")
+		  ;;(concat "\\(" labelholder "\\)\\|\\(" boundary "\\)")
+		  labelholder
 		  (regexp-quote YaTeX-comment-prefix)
-		  nil 1))
-	       (match-beginning 1))
+		  boundary 1))
+	       (match-beginning 0))
 	  ;; if \label{hoge} found, return it
 	  (buffer-substring
 	   (progn
@@ -507,7 +577,8 @@ Make \\label{xx} if no label."
 	   (progn
 	     (forward-sexp 1) (1- (point))))
 	;;else make a label
-	(goto-char (match-beginning 0))
+	;(goto-char (match-beginning 0))
+	(goto-char inspoint)
 	(skip-chars-backward " \t\n")
 	(save-excursion (setq newlabel (YaTeX::ref-generate-label)))
 	(delete-region (point) (progn (skip-chars-backward " \t") (point)))
@@ -519,7 +590,7 @@ Make \\label{xx} if no label."
 
 (defvar YaTeX::ref-labeling-regexp-alist
   '(("\\\\begin{java}{\\([^}]+\\)}" . 1)
-    ("\\\\elabel{\\([^}]+\\)}" . 1)))
+    ("\\\\label{\\([^}]+\\)}" . 1)))
 (defvar YaTeX::ref-labeling-regexp
   (mapconcat 'car YaTeX::ref-labeling-regexp-alist "\\|"))
 (defvar YaTeX::ref-mathenv-regexp
@@ -555,7 +626,7 @@ YaTeX-sectioning-level̐lŎw.")
 			       (car s))))
 			YaTeX-sectioning-level))
 		 "\\|")
-		"\\|caption\\){"
+		"\\|caption\\(\\[[^]]+\\]\\)?\\|footnote\\){"
 		"\\|\\(begin{\\(" mathenvs "\\|" enums  "\\)\\)\\)")))
 	  (regexp (concat "\\(" counter
 			  "\\)\\|\\(" YaTeX::ref-labeling-regexp "\\)"))
@@ -603,6 +674,7 @@ YaTeX-sectioning-level̐lŎw.")
 	      ;(goto-char (match-beginning 0))
 	      (setq e0 (match-end 0))
 	      (cond
+	       ((YaTeX-literal-p) nil)
 	       ((YaTeX-match-string 1)
 		;;if standard counter commands found 
 		(setq cmd (YaTeX-match-string 2))
@@ -646,15 +718,21 @@ YaTeX-sectioning-level̐lŎw.")
 				   (skip-chars-backward " \t")
 				   (1- (point)))
 			  (point-end-of-line)))
-		       x))))
-
+		       x))
+		    (setq e0 (point-max))))
+		 ;;else, simple section-type counter
 		 ((= (char-after (1- (point))) ?{)
 		  (setq label (buffer-substring
 			       (match-beginning 0)
 			       (progn (forward-char -1)
 				      (forward-list 1)
 				      (point))))
-		  (funcall output label match-point))
+		  (funcall output label match-point)
+		  ;; Skip preceding label if exists
+		  (if (save-excursion
+			(skip-chars-forward "\t \n")
+			(looking-at YaTeX::ref-labeling-regexp))
+		      (setq e0 (match-end 0))))
 		 (t
 		  (skip-chars-forward " \t")
 		  (setq label (buffer-substring
@@ -668,7 +746,10 @@ YaTeX-sectioning-level̐lŎw.")
 				     (1- (point)))
 				 (point-end-of-line))))
 		  (funcall output label match-point)
-		  ))
+		  (if (save-excursion
+			(skip-chars-forward "\t \n")
+			(looking-at YaTeX::ref-labeling-regexp))
+		      (setq e0 (match-end 0)))))
 		) ;;put label buffer
 	       ;;
 	       ;; if user defined label found
@@ -774,9 +855,13 @@ and print them to standard output."
 				      (point)))))))))
 	(setq files (cdr files))))))
 
+(defvar YaTeX::cite-bibitem-macro-regexp "bibitem\\|harvarditem"
+  "*Regexp of macro name of bibitem definition")
+
 (defun YaTeX::cite-collect-bibs-internal ()
   "Collect bibentry in the current buffer and print them to standard output."
-  (let ((ptn (concat YaTeX-ec-regexp "bibitem\\b"))
+  (let ((ptn (concat YaTeX-ec-regexp
+		     "\\(" YaTeX::cite-bibitem-macro-regexp "\\)\\b"))
 	(pcnt (regexp-quote YaTeX-comment-prefix)))
     (save-excursion
       (while (YaTeX-re-search-active-forward ptn pcnt nil t)
@@ -824,11 +909,22 @@ and print them to standard output."
 	      (YaTeX::cite-collect-bibs-internal)))
 	;;Now bbuf holds the list of bibitem
 	(set-buffer bbuf)
-	(YaTeX::ref argp "\\\\\\(bibitem\\)\\(\\[.*\\]\\)?" "cite"))))
-  
+	(YaTeX::ref
+	 argp 
+	 (concat "\\\\\\("
+		 YaTeX::cite-bibitem-macro-regexp
+		 "\\)\\(\\[.*\\]\\)?")
+	 "cite"))))
+
    (t nil)))
 
+;;; for AMS-LaTeX
 (and YaTeX-use-AMS-LaTeX (fset 'YaTeX::eqref 'YaTeX::ref))
+;;; for Harvard citation style
+(fset 'YaTeX::citeasnoun 'YaTeX::cite)
+(fset 'YaTeX::possessivecite 'YaTeX::cite)
+(fset 'YaTeX::citeyear 'YaTeX::cite)
+(fset 'YaTeX::citename 'YaTeX::cite)
 
 (defun YaTeX-yatex-buffer-list ()
   (save-excursion
@@ -1246,6 +1342,27 @@ and print them to standard output."
 	(format "{%s}[%s]" vmag hmag)
       (format "{%s}" vmag))))
 
+(defun YaTeX:includegraphics ()
+  "Add-in for \\includegraphics's option"
+  (let (width height (scale "") angle str)
+    (setq width (read-string "Width: ")
+	  height (read-string "Height: "))
+    (or (string< width "") (string< "" height)
+	(setq scale (read-string "Scale: ")))
+    (setq angle (read-string "Angle(0-359): "))
+    (setq str
+	  (mapconcat
+	   'concat
+	   (delq nil
+		 (mapcar '(lambda (s)
+			    (and (stringp (symbol-value s))
+				 (string< "" (symbol-value s))
+				 (format "%s=%s" s (symbol-value s))))
+			 '(width height scale angle)))
+	   ","))
+    (if (string= "" str) ""
+      (concat "[" str "]"))))
+
 (defun YaTeX::includegraphics (argp)
   "Add-in for \\includegraphics"
   (cond
