@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX environment-specific functions.
 ;;; yatexenv.el
-;;; (c ) 1994-1997 by HIROSE Yuuji.[yuuji@yatex.org]
-;;; Last modified Thu Aug 26 18:14:51 1999 on firestorm
-;;; $Id: yatexenv.el,v 1.68 2000/04/09 03:38:09 yuuji Rel $
+;;; (c ) 1994-2003 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Sun Mar 30 19:25:47 2003 on firestorm
+;;; $Id: yatexenv.el,v 1.71 2003/05/01 13:38:49 yuuji Rel $
 
 ;;;
 ;; Functions for tabular environment
@@ -230,7 +230,7 @@ Return the list of (No.ofCols PointEndofFormat)"
 	  (looking-at "\\\\right\\b"))
 	(progn (YaTeX-reindent
 		(save-excursion (YaTeX-goto-corresponding-leftright)
-				(1- (current-column)))))
+				(- (current-column) 0))))
       (save-excursion
 	(forward-line -1)
 	(while (and (not (bobp)) (YaTeX-on-comment-p))
@@ -280,12 +280,11 @@ Return the list of (No.ofCols PointEndofFormat)"
 
 ;;;###autoload
 (defun YaTeX-goto-corresponding-leftright ()
-  "Go to corresponding \left or \right.
-Note that this function assumes the corresponding \left\right
-is on another line."
+  "Go to corresponding \left or \right."
   (let ((YaTeX-struct-begin "\\left%1")
 	(YaTeX-struct-end "\\right%1")
-	(YaTeX-struct-name-regexp "[][(){}.|]"))
+	(YaTeX-struct-name-regexp "[][(){}\\.|]")
+	(in-leftright-p t))
     (YaTeX-goto-corresponding-environment t)))
 
 ;;;
