@@ -1,9 +1,9 @@
 ;;; -*- Emacs-Lisp -*-
 ;;; YaTeX process handler.
 ;;; yatexprc.el
-;;; (c )1993-2000 by HIROSE Yuuji.[yuuji@yatex.org]
-;;; Last modified Mon Dec 25 19:18:19 2000 on firestorm
-;;; $Id: yatexprc.el,v 1.69 2000/12/25 10:19:50 yuuji Rel $
+;;; (c )1993-2003 by HIROSE Yuuji.[yuuji@yatex.org]
+;;; Last modified Thu May  1 22:37:31 2003 on firestorm
+;;; $Id: yatexprc.el,v 1.71 2003/05/01 13:38:49 yuuji Rel $
 
 ;(require 'yatex)
 (require 'yatexlib)
@@ -221,8 +221,10 @@ operation to the region."
        preamble
        (if (re-search-forward "^[ 	]*\\\\begin.*{document}" nil t)
 	   (buffer-substring (point-min) (match-end 0))
-	 (concat "\\documentstyle{" YaTeX-default-document-style "}\n"
-		 "\\begin{document}")))
+	 (concat
+	  (if YaTeX-use-LaTeX2e "\\documentclass{" "\\documentstyle{")
+	  YaTeX-default-document-style "}\n"
+	  "\\begin{document}")))
       (goto-char opoint)
       ;;(set-buffer buffer)		;for clarity
       (let ((hilit-auto-highlight nil))
@@ -384,18 +386,20 @@ PROC should be process identifier."
 (defun YaTeX-get-paper-type ()
   "Search options in header and return a paper type, such as \"a4\", \"a4r\", etc."
   (save-excursion
+    (YaTeX-visit-main t)
     (goto-char (point-min))
-    (if (re-search-forward
-	 "^[ \t]*\\\\document\\(style\\|class\\)[ \t]*\\[\\([^]]*\\)\\]" nil t)
-	(let ((opts (YaTeX-split-string (YaTeX-match-string 2) "[ \t]*,[ \t]*")))
-	  (concat
-	   (catch 'found-paper
-	     (mapcar (lambda (pair)
-		       (if (member (car pair) opts)
-			   (throw 'found-paper (cdr pair))))
-		     YaTeX-paper-type-alist)
-	     YaTeX-default-paper-type)
-	   (if (member "landscape" opts) "r" ""))))))
+    (let ((opts
+	   (if (re-search-forward
+		"^[ \t]*\\\\document\\(style\\|class\\)[ \t]*\\[\\([^]]*\\)\\]" nil t)
+	       (YaTeX-split-string (YaTeX-match-string 2) "[ \t]*,[ \t]*"))))
+      (concat
+       (catch 'found-paper
+	 (mapcar (lambda (pair)
+		   (if (member (car pair) opts)
+		       (throw 'found-paper (cdr pair))))
+		 YaTeX-paper-type-alist)
+	 YaTeX-default-paper-type)
+       (if (member "landscape" opts) "r" "")))))
 
 (defvar YaTeX-preview-command-history nil
   "Holds minibuffer history of preview command.")
